@@ -9,13 +9,12 @@ from .._logging import get_logger
 
 logger = get_logger()
 
-from ..lib import fileio, pairsam_format, headerops
-from . import cli, common_io_options
-
 import click
 
+from ..lib import fileio, headerops, pairsam_format
 from ..lib.dedup import streaming_dedup, streaming_dedup_cython
 from ..lib.stats import PairCounter
+from . import cli, common_io_options
 
 UTIL_NAME = "pairtools_dedup"
 
@@ -105,7 +104,7 @@ UTIL_NAME = "pairtools_dedup"
     " It is available for backwards compatibility and to allow specification of the"
     " column order."
     " Now the default scipy backend is generally the fastest, and with chunksize below"
-    " 1 mln has the lowest memory requirements. [dedup option]"
+    " 1 mln has the lowest memory requirements. [dedup option]",
     # " 'cython' is deprecated and provided for backwards compatibility",
 )
 
@@ -140,10 +139,11 @@ UTIL_NAME = "pairtools_dedup"
 
 ### Output options:
 @click.option(
-    "--mark-dups",
+    "--mark-dups/--no-mark-dups",
+    default=True,
     is_flag=True,
-    help='If specified, duplicate pairs are marked as DD in "pair_type" and '
-    "as a duplicate in the sam entries. [output format option]",
+    help='Specify if duplicate pairs should be marked as DD in "pair_type" and '
+    "as a duplicate in the sam entries. True by default. [output format option]",
 )
 @click.option(
     "--keep-parent-id",
@@ -179,45 +179,45 @@ UTIL_NAME = "pairtools_dedup"
 )
 @click.option(
     "--c1",
-    type=int,
-    default=pairsam_format.COL_C1,
-    help=f"Chrom 1 column; default {pairsam_format.COL_C1}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[1],
+    help=f"Chrom 1 column; default {pairsam_format.COLUMNS_PAIRS[1]}"
+    "[input format option]",
 )
 @click.option(
     "--c2",
-    type=int,
-    default=pairsam_format.COL_C2,
-    help=f"Chrom 2 column; default {pairsam_format.COL_C2}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[3],
+    help=f"Chrom 2 column; default {pairsam_format.COLUMNS_PAIRS[3]}"
+    "[input format option]",
 )
 @click.option(
     "--p1",
-    type=int,
-    default=pairsam_format.COL_P1,
-    help=f"Position 1 column; default {pairsam_format.COL_P1}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[2],
+    help=f"Position 1 column; default {pairsam_format.COLUMNS_PAIRS[2]}"
+    "[input format option]",
 )
 @click.option(
     "--p2",
-    type=int,
-    default=pairsam_format.COL_P2,
-    help=f"Position 2 column; default {pairsam_format.COL_P2}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[4],
+    help=f"Position 2 column; default {pairsam_format.COLUMNS_PAIRS[4]}"
+    "[input format option]",
 )
 @click.option(
     "--s1",
-    type=int,
-    default=pairsam_format.COL_S1,
-    help=f"Strand 1 column; default {pairsam_format.COL_S1}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[5],
+    help=f"Strand 1 column; default {pairsam_format.COLUMNS_PAIRS[5]}"
+    "[input format option]",
 )
 @click.option(
     "--s2",
-    type=int,
-    default=pairsam_format.COL_S2,
-    help=f"Strand 2 column; default {pairsam_format.COL_S2}"
-    " Only works with '--backend cython'. [input format option]",
+    type=str,
+    default=pairsam_format.COLUMNS_PAIRS[6],
+    help=f"Strand 2 column; default {pairsam_format.COLUMNS_PAIRS[6]}"
+    "[input format option]",
 )
 @click.option(
     "--unmapped-chrom",
@@ -486,12 +486,14 @@ def dedup_py(
             "Pairs file appears not to be sorted, dedup might produce wrong results."
         )
     header = headerops.append_new_pg(header, ID=UTIL_NAME, PN=UTIL_NAME)
+    dups_header = header.copy()
+    if keep_parent_id and len(dups_header) > 0:
+        dups_header = headerops.append_columns(dups_header, ["parent_readID"])
+    if outstream == outstream_dups:
+        header = dups_header
     if send_header_to_dedup:
         outstream.writelines((l + "\n" for l in header))
     if send_header_to_dup and outstream_dups and (outstream_dups != outstream):
-        dups_header = header
-        if keep_parent_id and len(dups_header) > 0:
-            dups_header = headerops.append_columns(dups_header, ["parent_readID"])
         outstream_dups.writelines((l + "\n" for l in dups_header))
     if (
         outstream_unmapped
@@ -505,8 +507,8 @@ def dedup_py(
     extra_cols2 = []
     if extra_col_pair is not None:
         for col1, col2 in extra_col_pair:
-            extra_cols1.append(column_names[col1] if col1.isdigit() else col1)
-            extra_cols2.append(column_names[col2] if col2.isdigit() else col2)
+            extra_cols1.append(column_names[col1] if col1.isnumeric() else col1)
+            extra_cols2.append(column_names[col2] if col2.isnumeric() else col2)
 
     if backend == "cython":
         # warnings.warn(
@@ -516,6 +518,12 @@ def dedup_py(
         # )
         extra_cols1 = [column_names.index(col) for col in extra_cols1]
         extra_cols2 = [column_names.index(col) for col in extra_cols2]
+        c1 = column_names.index(c1)
+        c2 = column_names.index(c2)
+        p1 = column_names.index(p1)
+        p2 = column_names.index(p2)
+        s1 = column_names.index(s1)
+        s2 = column_names.index(s2)
         streaming_dedup_cython(
             method,
             max_mismatch,
@@ -555,6 +563,12 @@ def dedup_py(
             out_stat=out_stat,
             backend=backend,
             n_proc=n_proc,
+            c1=c1,
+            c2=c2,
+            p1=p1,
+            p2=p2,
+            s1=s1,
+            s2=s2,
         )
     else:
         raise ValueError("Unknown backend")
@@ -564,9 +578,9 @@ def dedup_py(
         out_stat.save(
             out_stats_stream,
             yaml=kwargs.get("yaml", False),  # format as yaml
-            filter=first_filter_name
-            if not kwargs.get("yaml", False)
-            else None,  # output only the first filter if non-YAML output
+            filter=(
+                first_filter_name if not kwargs.get("yaml", False) else None
+            ),  # output only the first filter if non-YAML output
         )
 
     if bytile_dups:
