@@ -1,8 +1,8 @@
 from . import fileio
 from . import dedup
-from . import dedup_cython
 from . import filterbycov
 from . import headerops
+from . import pairsio
 from . import pairsam_format
 from . import parse
 from . import parse_pysam
