@@ -2,7 +2,7 @@
 import os
 import sys
 
-from nose.tools import assert_raises
+import pytest
 
 import subprocess
 
