@@ -22,4 +22,6 @@ def test_scaling():
 
     output = pd.read_csv(io.StringIO(result), sep="\t", header=0)
 
-    assert output["n_pairs"].sum() == 5
+    assert (
+        output["n_pairs"].sum() == 9
+    )  # double unmapped pairs are currently ignored by lib.scaling
